#include "utils.PRAP"
#include "Poli.hpp"
#include <vector>

/* Constantes */

const char *ER1 = "Polinomio demasiado grande";
const char *ER2 = "Exponente negativo";
const char *ER3 = "Exponente demasiado grande";

/* Hemos dejado algunos this no necesarios como recordatorio */ 
/* Tambien se lanzan algunas excepciones a modo de ejemplo */


/* Constructoras */

Poli::Poli()
{
}

// leer como constructora
Poli::Poli(int marca)

  /* Lee los coeficientes de un polinomio de menor a mayor indice,
     hasta llegar a la marca.
     Si se leen demasiados coeficientes se lanza una excepcion */
{
    crear_poli(0, 0);
    int x = readint();
    int i = 0;
    while (x != marca) {
	if (i > grado_maximo())
	    throw PRAPExcepcio(ER1);
	modif_coef(x, i);
	x = readint();
	i++;
    }
}

/* Destructora por defecto */

Poli::~Poli()
{
}

int Poli::grado_maximo() const
  /* Para consultar el maximo grado permitido por la implementacion */
  /* Unico ejemplo hasta ahora de metodo estatico en una clase de datos */
{
    return MAX_GRADO;
}

void Poli::crear_poli(int c, int i)
  /* i ha de estar entre 0 y MAX_GRADO */
  /* El parametro implicito es cx^i */
{
    // ejemplos de definicion de excepciones: control del parametro i

    if (i < 0)
	throw PRAPExcepcio(ER2);
    if (i > MAX_GRADO)
	throw PRAPExcepcio(ER3);

    // si esta todo correcto, se inicializan los coeficientes;
    // todos a 0, excepto el i-esimo

    coefs = vector < int >(MAX_GRADO + 1);
    int j = 0;
    while (j <= MAX_GRADO) {
	coefs[j] = 0;
	j++;
    }
    coefs[i] = c;
    grau = i;
}

void Poli::modif_coef(int c, int i)
  /* i ha de estar entre 0 y MAX_GRADO */
  /* El parametro implicito cambia su termino i-simo a cx^i */
{
    // control del parametro i

    if (i < 0)
	throw PRAPExcepcio(ER2);
    if (i > MAX_GRADO)
	throw PRAPExcepcio(ER3);

    // si esta todo correcto, se modifica el coeficiente i-esimo;

    coefs[i] = c;

    // hay que controlar si el grado sube

    if (grau < i && c != 0)
	grau = i;

    // o baja

    if (grau == i && c == 0)
	while (grau > 0 && coefs[grau] == 0)
	    grau--;
}

void Poli::suma_poli(Poli & p)
  /* cierto */
  /* El parametro implicito pasa a ser la suma de su valor antiguo y p */
{
    // si el grado de p es mayor que el del parametro implicito,
    // hay que cambiar este

    if (p.grau > this->grau) {
	this->grau = p.grau;
    }
    // sumas de coeficientes; notad que loe coeficientes 
    // superiores al grado no cambian

    int i = 0;
    while (i <= this->grau) {
	this->coefs[i] += p.coefs[i];
	i++;
    }

    // si se han anulado algunos de los coefs de mayor grado
    // el grado decrece

    while (this->grau > 0 && this->coefs[this->grau] == 0)
	this->grau--;
}

int Poli::grado() const
  /* cierto */
  /* retorna el grado del parametro implicito */
{
    return grau;
}

int Poli::coef(int i) const
  /* i ha de estar entre 0 y el grado del parametro implicito */
  /* retorna el coeficiente i-esimo del parametro implicito */
{
    if (i < 0)
	throw PRAPExcepcio(ER2);
    if (i > grau)
	throw PRAPExcepcio(ER3);
    return coefs[i];
}


    // leer como modificadora

void Poli::leer_poli(int marca)

  /* Lee los coeficientes de un polinomio de menor a mayor indice,
     hasta llegar a la marca.
     Si se leen demasiados coeficientes se lanza una excepcion */
{
    crear_poli(0, 0);
    int x = readint();
    int i = 0;
    while (x != marca) {
	if (i > grado_maximo())
	    throw PRAPExcepcio(ER1);
	modif_coef(x, i);
	x = readint();
	i++;
    }
}

    // escribir

void Poli::escribir_poli() const
  /* Escribe los coeficientes de un polinomio de menor a mayor indice */
{
    int i = 0;
    while (i <= grado()) {
	cout << coef(i) << " ";
	i++;
    }
    cout << endl;
}
